package cz.frantovo.hrisniciSpameri;

import cz.frantovo.hrisniciSpameri.dao.CilDAO;
import cz.frantovo.hrisniciSpameri.dao.SouhrnDAO;
import cz.frantovo.hrisniciSpameri.dto.Cil;
import cz.frantovo.hrisniciSpameri.dto.Souhrn;
import java.io.Serializable;
import java.sql.Date;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Beana zprostředkovávající denní souhrn hříšníků.
 * @author fiki
 */
public class DenniSouhrn implements Serializable {

    private static final long serialVersionUID = 7932392366943861342L;
    private static final Logger log = Logger.getLogger(DenniSouhrn.class.getName());
    private static String formatDataVzor = "yyyy-MM-dd";
    public static DateFormat formatData = new SimpleDateFormat(formatDataVzor);
    private Date zacatek;
    private Date konec;
    private int cil;

    public String getZacatekString() {
        return formatData.format(getZacatek());
    }

    public Date getZacatek() {
        if (zacatek == null) {
            nastavVychoziZacatek();
        }
        return zacatek;
    }

    public void setZacatek(Date zacatek) {
        if (zacatek == null) {
            nastavVychoziZacatek();
        } else {
            this.zacatek = zacatek;
        }
        log.log(Level.FINE, "Začátek nastaven na: " + formatData.format(zacatek));
    }

    public void setZacatekString(String zacatek) {
        if (zacatek == null) {
            nastavVychoziZacatek();
        } else {
            try {
                this.zacatek = new Date(formatData.parse(zacatek).getTime());
            } catch (ParseException e) {
                log.log(Level.FINE, "Chybně zadané datum – začátek: " + zacatek, e);
                nastavVychoziZacatek();
            }
        }
    }

    public Date getKonec() {
        if (konec == null) {
            nastavVychoziKonec();
        }
        return konec;
    }

    public void setKonec(Date konec) {
        if (konec == null) {
            nastavVychoziKonec();
        } else {
            this.konec = konec;
        }
        log.log(Level.FINE, "Konec nastaven na: " + formatData.format(zacatek));
    }

    public void setKonecString(String konec) {
        if (konec == null) {
            nastavVychoziKonec();
        } else {
            try {
                this.konec = new Date(formatData.parse(konec).getTime());
            } catch (Exception e) {
                nastavVychoziKonec();
            }
        }
    }

    public void setCilString(String cil) {
        try {
            this.cil = Integer.parseInt(cil);
        } catch (Exception e) {
            this.cil = 0;
        }
    }

    public void setCil(int cil) {
        this.cil = cil;
    }

    public String getCilString() {
        return String.valueOf(cil);
    }

    public int getCil() {
        return cil;
    }

    private void nastavVychoziZacatek() {
        log.log(Level.FINE, "Nastavuji začátek na výchozí hodnotu.");
        setZacatek(getVychoziZacatek());
    }

    private void nastavVychoziKonec() {
        log.log(Level.FINE, "Nastavuji konec na výchozí hodnotu.");
        setKonec(getVychoziKonec());
    }

    public Collection<Souhrn> getSouhrn() {
        Collection<Souhrn> souhrn = new SouhrnDAO().getSouhrn(zacatek, konec, cil);

        /** ne všechno musíme posílat do JSP vrstvy */
        for (Souhrn s : souhrn) {
            s.getCil().setDatabaze(null);
            s.getCil().setSelekt(null);
        }

        return souhrn;
    }

    /**
     * @return Seznam všech cílů. Ale bez těchto hodnot: databáze a selekt.
     */
    public Collection<Cil> getSeznamCilu() {
        Collection<Cil> cile = new CilDAO().getCile();

        /** ne všechno musíme posílat do JSP vrstvy */
        for (Cil c : cile) {
            c.setDatabaze(null);
            c.setSelekt(null);
        }

        return cile;
    }

    /**
     * @return Výchozí začátek je před měsícem
     */
    private static Date getVychoziZacatek() {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);
        return new Date(c.getTimeInMillis());
    }

    /**
     * @return  Výchozí konec je současnost
     */
    private static Date getVychoziKonec() {
        return new Date(System.currentTimeMillis());
    }
}
